@@ -218,19 +218,13 @@ class StoryTextSWTBotGefFigureCanvas(gefbot.widgets.SWTBotGefFigureCanvas):
         
     def mouseMoveLeftClick(self, x, y, keyModifiers=0):
         displayLoc = swtsimulator.runOnUIThread(self.toDisplayLocation, x, y)
-        currPos = swtsimulator.runOnUIThread(self.getCursorLocation)
-        self.eventPoster.moveMouseAndWait(displayLoc.x, displayLoc.y)
-        self.eventPoster.checkAndPostKeyPressed(keyModifiers)
-        self.eventPoster.clickMouse()
-        self.eventPoster.checkAndPostKeyReleased(keyModifiers)
-        self.eventPoster.moveMouseAndWait(currPos.x, currPos.y)
+        self.eventPoster.moveClickAndReturn(displayLoc.x, displayLoc.y, keyModifiers)
                 
     def toDisplayLocation(self, x, y):
         return self.widget.getDisplay().map(self.widget, None, x, y)
 
-    def getCursorLocation(self):
-        return self.widget.getDisplay().getCursorLocation()
-   
+    def isDisposed(self):
+        return self.widget.isDisposed()
 
 class DisplayFilter(rcpsimulator.DisplayFilter):
     def shouldCheckWidget(self, widget, eventType):
@@ -259,11 +253,9 @@ class WidgetMonitor(rcpsimulator.WidgetMonitor):
         for viewer in self.getViewers(botView):
             menu = viewer.getViewer().getControl().getMenu()
             if menu:
+                rcpsimulator.WidgetAdapter.storeId(menu, botView.getViewReference().getId())
                 self.monitorMenu(menu)
-
-    def sendShowEvent(self, menu):
-        menu.notifyListeners(swt.SWT.Show, swt.widgets.Event())
-        
+                    
     def monitorMenu(self, menu):
         if menu.getItemCount() == 0:
             # The menu is a Contribution defined in plugin.xml. We have to send an extra
@@ -317,21 +309,21 @@ class GefViewerAdapter(rcpsimulator.WidgetAdapter):
         self.partReference = partRef
         rcpsimulator.WidgetAdapter.__init__(self, widget)
 
-    def getUIMapIdentifier(self):
-        partId = self.partReference.getId()
-        if isinstance(self.partReference, EditorReference):
-            return self.encodeToLocale("Editor=" + partId + ", " +"Viewer")
-        else:
-            return self.encodeToLocale("View=" + partId + ", " +"Viewer")
-
     def findPossibleUIMapIdentifiers(self):
-        return [ self.getUIMapIdentifier() ]
+        partId = self.partReference.getId()
+        return [ "Type=" + self.getType(), "View=" + partId ]
     
     def getType(self):
         return "Viewer"
     
     def getPartName(self):
         return self.partReference.getPartName()
+    
+    def isPreferred(self):
+        # Ensure we replay for the one in the active view, if this is possible
+        # Same object (identifier) may exist in several views
+        page = self.partReference.getPage()
+        return page.getActivePart() == self.partReference.getPart(False)
 
 class ViewerEvent(storytext.guishared.GuiEvent):
     def __init__(self, *args, **kw):
@@ -354,7 +346,7 @@ class ViewerEvent(storytext.guishared.GuiEvent):
                 val = int(intVal)
                 return desc[:startPos] + str(val + 1) + ")"
         return desc + " (2)"
-
+    
     def storeObjectDescription(self, part, checkParent=True):
         if part in self.allDescriptions:
             oldDesc = self.allDescriptions.get(part)
@@ -428,6 +420,14 @@ class ViewerEvent(storytext.guishared.GuiEvent):
     def getEditPartComparator(self):
         return None
 
+    def getControl(self):
+        return self.getGefViewer().getControl()
+
+    def widgetDisposed(self):
+        return self.widget.getFigureCanvas().isDisposed()
+    
+    def describeWidget(self):
+        return "of type " + self.widget.getType()
 
 class ViewerSelectEvent(ViewerEvent):
     def connectRecord(self, method):
@@ -455,8 +455,11 @@ class ViewerSelectEvent(ViewerEvent):
                 return self.makePartialParseFailure(parts, ",".join(notfound), allParts[0] not in notfound)
             else:
                 return parts
-        else:
-            raise UseCaseScriptError, "could not find any objects in viewer matching description "  + repr(description)
+        elif len(allParts) > 1 and len(notfound) > 0:
+            editPart = self.findEditPart(self.widget.rootEditPart(), description)
+            if editPart:
+                return [ editPart ]
+        raise UseCaseScriptError, "could not find any objects in viewer matching description '"  + description + "'"
         
     def generate(self, parts, partial=False):
         if not partial:
@@ -483,7 +486,7 @@ class ViewerSelectEvent(ViewerEvent):
 
     def shouldRecord(self, parts, *args):
         types = self.getSignalsToFilter()
-        hasMouseEvent = DisplayFilter.instance.hasEventOfType(types, self.getGefViewer().getControl())
+        hasMouseEvent = DisplayFilter.instance.hasEventOfType(types, self.getControl())
         return hasMouseEvent and len(parts) > 0
 
     def implies(self, stateChangeOutput, stateChangeEvent, *args):
@@ -510,7 +513,7 @@ class ViewerDragAndDropEvent(ViewerEvent):
             def dragDetected(lself, event):#@NoSelf
                 storytext.guishared.catchAll(self.applyToDragged, event, method)
 
-        rcpsimulator.swtsimulator.runOnUIThread(self.getGefViewer().getControl().addDragDetectListener, DDListener())
+        rcpsimulator.swtsimulator.runOnUIThread(self.getControl().addDragDetectListener, DDListener())
         self.addDropListener(method)
 
     def applyToDragged(self, event, method):
@@ -525,14 +528,14 @@ class ViewerDragAndDropEvent(ViewerEvent):
                 storytext.guishared.catchAll(self.handleDrop, event, method)
 
         self.allInstances.setdefault(self.__class__, []).append(self)
-        rcpsimulator.swtsimulator.runOnUIThread(self.getGefViewer().getControl().addMouseListener, MListener())
+        rcpsimulator.swtsimulator.runOnUIThread(self.getControl().addMouseListener, MListener())
 
     def handleDrop(self, event, method):
         if DragHolder.sourceEvent is not None and self.__class__ is DragHolder.sourceEvent.__class__:
             for instance in self.allInstances.get(self.__class__, []):
             # No guarantee MouseUp appers on same widget
                 dropX, dropY = instance.getDropPosition(event)
-                if dropX is not None and instance.isValidDragAndDrop(DragHolder.sourceSwtEvent, event, dropX, dropY):
+                if dropX is not None and instance.getControl().isVisible() and instance.isValidDragAndDrop(DragHolder.sourceSwtEvent, event, dropX, dropY):
                     method(DragHolder.draggedPart, dropX, dropY, instance, DragHolder.sourceEvent)
                     break
             DragHolder.reset()
@@ -561,7 +564,7 @@ class ViewerDragAndDropEvent(ViewerEvent):
     
     def shouldRecord(self, part, *args):
         types = self.getSignalsToFilter()
-        return DisplayFilter.instance.hasEventOfType(types, self.getGefViewer().getControl())
+        return DisplayFilter.instance.hasEventOfType(types, self.getControl())
 
     def generate(self, partAndPos):
         self.widget.drag(*partAndPos)
@@ -570,7 +573,7 @@ class ViewerDragAndDropEvent(ViewerEvent):
         sourceDesc, dest = description.split(" to ", 1)
         editPart = self.findEditPart(self.widget.rootEditPart(), sourceDesc)
         if not editPart:
-            raise UseCaseScriptError, "could not find any objects in viewer matching description " + repr(sourceDesc)
+            raise UseCaseScriptError, "could not find any objects in viewer matching description '" + sourceDesc + "'"
         
         displayLocation = self.parseDestination(dest)
         return editPart, displayLocation.x, displayLocation.y
