@@ -2,10 +2,11 @@
 """ Simulation stuff specific to using Eclipse RCP. For example View IDs and Editor IDs etc."""
 
 from storytext.javaswttoolkit import simulator as swtsimulator
+from storytext.javaswttoolkit import util
 import storytext.guishared
 from storytext.definitions import UseCaseScriptError
 import org.eclipse.swtbot.eclipse.finder as swtbot
-from org.eclipse.ui import IPartListener, IPropertyListener, IWorkbenchPartConstants
+from org.eclipse.ui import PlatformUI, IPartListener, IPropertyListener, IWorkbenchPartConstants
 
 # If classes get mentioned for the first time in the event dispatch thread, they will get the wrong classloader
 # So we load everything we'll ever need here, once, where we know what the classloader is
@@ -20,29 +21,26 @@ from org.eclipse.swt import SWT
 from org.eclipse.ui.forms.widgets import ExpandableComposite
 from org.eclipse.ui.forms.events import ExpansionAdapter
 from org.eclipse.swtbot.swt.finder.widgets import AbstractSWTBotControl
-from java.lang import NullPointerException
+from org.eclipse.jface.bindings.keys import KeyStroke 
+from java.lang import NullPointerException, Integer
 
 class WidgetAdapter(swtsimulator.WidgetAdapter):
-    widgetViewIds = {}
-    def getUIMapIdentifier(self):
-        orig = swtsimulator.WidgetAdapter.getUIMapIdentifier(self)
-        if orig.startswith("Type="):
-            return self.addViewId(self.widget.widget, orig)
+    widgetViewIds = {}    
+    swtsimulator.WidgetAdapter.secondaryIdentifiers.append("View")
+    def getViewWidget(self):
+        widget = self.widget.widget
+        if isinstance(widget, MenuItem):
+            return swtsimulator.runOnUIThread(util.getRootMenu, widget)
         else:
-            return orig
+            return widget
 
     def findPossibleUIMapIdentifiers(self):
         orig = swtsimulator.WidgetAdapter.findPossibleUIMapIdentifiers(self)
-        orig[-1] = self.addViewId(self.widget.widget, orig[-1])
+        viewWidget = self.getViewWidget()
+        if viewWidget in self.widgetViewIds:
+            orig.append("View=" + self.widgetViewIds.get(viewWidget))
         return orig
 
-    def addViewId(self, widget, text):
-        viewId = self.widgetViewIds.get(widget)
-        if viewId:
-            return "View=" + viewId + "," + text
-        else:
-            return text
-
     def getNameForAppEvent(self):
         name = self.getName()
         if name:
@@ -52,14 +50,37 @@ class WidgetAdapter(swtsimulator.WidgetAdapter):
             return viewId.lower().split(".")[-1]
         else:
             return self.getType().lower()
-
+        
+    @classmethod
+    def storeId(cls, widget, viewId):
+        cls.widgetViewIds[widget] = viewId
+        
     @classmethod
     def storeIdWithChildren(cls, widget, viewId):
-        cls.widgetViewIds[widget] = viewId
+        cls.storeId(widget, viewId)
         if hasattr(widget, "getChildren"):
             for child in widget.getChildren():
                 cls.storeIdWithChildren(child, viewId)
-
+        if hasattr(widget, "getMenu") and widget.getMenu():
+            cls.storeId(widget.getMenu(), viewId)
+        
+    def isPreferred(self):
+        viewWidget = self.getViewWidget()
+        if viewWidget in self.widgetViewIds:
+            viewId = self.widgetViewIds.get(viewWidget)
+            activeViewId = swtsimulator.runOnUIThread(self.getActiveViewId)
+            return viewId == activeViewId
+        else:
+            return False
+        
+    @staticmethod
+    def getActiveViewId():
+        try:
+            return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePartReference().getId()
+        except AttributeError, e:
+            if "NoneType" not in str(e):
+                raise
+        
 class DisplayFilter(swtsimulator.DisplayFilter):
     def hasComplexAncestors(self, widget):
         return self.isFilterTreeText(widget) or swtsimulator.DisplayFilter.hasComplexAncestors(self, widget)
@@ -77,13 +98,7 @@ class WidgetMonitor(swtsimulator.WidgetMonitor):
         self.allViews = set()
         self.swtbotMap[ExpandableComposite] = (SWTBotExpandableComposite, [])
         swtsimulator.WidgetMonitor.__init__(self, *args, **kw)
-        
-    def getWidgetEventTypes(self):
-        # Do this here, when things will be loaded with the right classloader
-        # Might affect which event types are used 
-        self.uiMap.scriptEngine.importCustomEventTypesFromSimulator()
-        return swtsimulator.WidgetMonitor.getWidgetEventTypes(self)
-        
+            
     def createSwtBot(self):
         return swtbot.SWTWorkbenchBot()
     
@@ -107,23 +122,33 @@ class WidgetMonitor(swtsimulator.WidgetMonitor):
             ref = swtbotView.getViewReference()
             if ref not in self.allViews:
                 self.allViews.add(ref)
-                viewparent = ref.getPane().getControl()
+                pane = ref.getPane()
+                viewparent = pane.getControl()
                 if viewparent:
                     self.uiMap.logger.debug("Caching View with ID " + ref.getId())
                     WidgetAdapter.storeIdWithChildren(viewparent, ref.getId())
+                toolbar = pane.getToolBar()
+                if toolbar:
+                    for item in toolbar.getItems():
+                        WidgetAdapter.storeId(item, ref.getId())
                 adapter = ViewAdapter(swtbotView)
                 self.uiMap.monitorWidget(adapter)
                 self.monitorMenus(swtbotView)
                 self.addTitleChangedListener(swtbotView)
+                self.addActivePartListener(swtbotView)
                 
-    def handleReplayFailure(self, errorText, *args):
-        if "MenuItem has already been disposed" in errorText: # View Menus can easily get regenerated, try to re-monitor them.
-            swtsimulator.runOnUIThread(self.monitorAllMenus)
-    
-    def monitorAllMenus(self):
+    def addActivePartListener(self, swtbotView):
+        class RecordListener(IPartListener):
+            def partActivated(listenerSelf, part):#@NoSelf
+                self.monitorMenus(swtbotView)
+        page = swtbotView.getViewReference().getPage()
+        swtsimulator.runOnUIThread(page.addPartListener, RecordListener())
+                
+    def recheckPopupMenus(self):
         for swtbotView in self.getViews():
             self.uiMap.logger.debug("Menu item disposed - remonitoring menu in view " + swtbotView.getViewReference().getId())
             self.monitorMenus(swtbotView)
+        swtsimulator.WidgetMonitor.recheckPopupMenus(self)
                 
     def setWidgetAdapter(self):
         WidgetAdapter.setAdapterClass(WidgetAdapter)
@@ -145,49 +170,126 @@ class WidgetMonitor(swtsimulator.WidgetMonitor):
         self.monitorViewMenus(botView)
         self.monitorViewContentsMenus(botView)
         
+    def sendShowEvent(self, menu):
+        menu.notifyListeners(SWT.Show, Event())
+        
     def monitorViewMenus(self, botView):
-        pane = botView.getViewReference().getPane()
+        ref = botView.getViewReference()
+        pane = ref.getPane()
         if pane.hasViewMenu():            
             menuManager = pane.getMenuManager()
-            menu = menuManager.createContextMenu(pane.getControl().getParent())
-            menuManager.updateAll(True)
-            menu.notifyListeners(SWT.Show, Event())
-    
+            if pane.getControl():
+                menu = menuManager.createContextMenu(pane.getControl().getParent())
+                menuManager.updateAll(True)
+                WidgetAdapter.storeId(menu, ref.getId())
+                self.sendShowEvent(menu)
+            
     def monitorViewContentsMenus(self, botView):
         pass
+    
+class KeyBindingListener(Listener):
+    def __init__(self):
+        self.bindings = {}
+        self.listening = False
+    
+    def mapKeyBinding(self, keyStroke, method, display, event):
+        if not self.listening:
+            self.listening = True
+            self.addInitialFilter(display)
+        self.bindings[(keyStroke.getModifierKeys(), keyStroke.getNaturalKey())] = method, event
+            
+    def addInitialFilter(self, display):
+        # Have to push to the front of the queue (cheating as we do!) because Eclipse's own listener swallows the event
+        filterTable = util.getPrivateField(display, "filterTable")
+        existingListeners = util.callPrivateMethod(filterTable, "getListeners", [ SWT.KeyDown ], [ Integer.TYPE ])
+        for existingListener in existingListeners:
+            display.removeFilter(SWT.KeyDown, existingListener)
+        display.addFilter(SWT.KeyDown, self)
+        for existingListener in existingListeners:
+            display.addFilter(SWT.KeyDown, existingListener)
+    
+    def handleEvent(self, e): #@NoSelf
+        binding = e.stateMask, e.keyCode
+        if binding in self.bindings:
+            method, event = self.bindings[binding]
+            storytext.guishared.catchAll(method, e, event)
+
+class RCPSelectEvent(swtsimulator.SelectEvent):
+    bindingListener = KeyBindingListener()
+    def connectRecord(self, method):
+        swtsimulator.SelectEvent.connectRecord(self, method)
+        widget = self.widget.widget.widget
+        if hasattr(widget, "getAccelerator"):
+            swtsimulator.runOnUIThread(self.connectRecordKeyBinding, widget, method)
+            
+    def connectRecordKeyBinding(self, widget, method):
+        text = self.widget.getText()
+        # Aim is to handle Eclipse RCP key binding mechanism, not SWT accelerators which work anyway
+        if "\t" in text and widget.getAccelerator() == 0:
+            keyBinding = text.split("\t")[-1].lower()
+            keyStroke = KeyStroke.getInstance(keyBinding)
+            self.bindingListener.mapKeyBinding(keyStroke, method, widget.getDisplay(), self)
+            
+    def shouldRecord(self, event, *args):
+        if event.type == SWT.KeyDown:
+            # Accelerators aren't active in dialogs
+            return not self.widgetDisposed() and event.widget.getShell() is self.widget.widget.widget.getParent().getShell()
+        else:
+            return swtsimulator.SelectEvent.shouldRecord(self, event, *args)
 
 class ViewAdapter(swtsimulator.WidgetAdapter):
-    def getUIMapIdentifier(self):
-        viewId = self.widget.getViewReference().getId()
-        return self.encodeToLocale("View=" + viewId)
-
     def findPossibleUIMapIdentifiers(self):
-        return [ self.getUIMapIdentifier() ]
-    
+        return [ "Type=View", "View=" + self.widget.getViewReference().getId() ]
+
     def getType(self):
         return "View"
 
-class PartActivateEvent(storytext.guishared.GuiEvent):
+    
+class PartActivateEvent(swtsimulator.SignalEvent):
     allInstances = []
     def __init__(self, *args, **kw):
-        storytext.guishared.GuiEvent.__init__(self, *args, **kw)
+        swtsimulator.SignalEvent.__init__(self, *args, **kw)
         self.allInstances.append(self)
         
     def connectRecord(self, method):
         class RecordListener(IPartListener):
             def partActivated(listenerSelf, part):#@NoSelf
-                storytext.guishared.catchAll(method, part, self)
+                if part is self.widget.getViewReference().getView(False):
+                    storytext.guishared.catchAll(method, part, self)
         page = self.widget.getViewReference().getPage()
         swtsimulator.runOnUIThread(page.addPartListener, RecordListener())
-        
+                
     def generate(self, *args):
         # The idea is to just do this, but it seems to cause strange things to happen
         #internally. So we do it outside SWTBot instead.
         #self.widget.setFocus()
-        page = self.widget.getViewReference().getPage()
-        view = self.widget.getViewReference().getView(False)
-        swtsimulator.runOnUIThread(page.activate, view)
+        swtsimulator.runOnUIThread(self.clickMatchingTab, *args)
+        #page = self.widget.getViewReference().getPage()
+        #view = self.widget.getViewReference().getView(False)
+        #swtsimulator.runOnUIThread(page.activate, view)
         
+    def getTabFolder(self):
+        for tab in self.widget.getViewReference().getPane().getTabList():
+            if hasattr(tab, "getItems"):
+                return tab
+            
+    def clickMatchingTab(self, argumentString):
+        tabfolder = self.getTabFolder()
+        for item in tabfolder.getItems():
+            if item.getText() == argumentString:
+                self.clickItem(item, tabfolder)
+                        
+    def getCenter(self, item):
+        bounds = item.getBounds()
+        return bounds.x + bounds.width / 2, bounds.y + bounds.height / 2 
+    
+    def clickItem(self, item, tabfolder):
+        x, y = self.getCenter(item)
+        displayLoc = tabfolder.toDisplay(x, y)
+        display = tabfolder.getDisplay()
+        # Move the mouse pointer back so we don't get accidental mouseovers...
+        swtsimulator.EventPoster(display).moveClickAndReturn(displayLoc.x, displayLoc.y)
+    
     def parseArguments(self, argumentString):
         # The idea is to just do this, but it seems to cause strange things to happen
         #internally. So we do it outside SWTBot instead.
@@ -195,15 +297,19 @@ class PartActivateEvent(storytext.guishared.GuiEvent):
         if self.getTitleArgument() == argumentString:
             return argumentString
         else:
-            raise UseCaseScriptError, "Could not find View named " + repr(argumentString) + " to activate."
+            raise UseCaseScriptError, "Could not find View named '" + argumentString + "' to activate.\n" + \
+                "Views are named " + repr([ str(i.getTitleArgument()) for i in self.allInstances ])
+            
 
     def getTitle(self):
-        return self.widget.getViewReference().getTitle()
+        return self.widget.getViewReference().getPartName()
 
     def shouldRecord(self, part, *args):
         # TODO: Need to check no other events are waiting in DisplayFilter 
-        return part is self.widget.getViewReference().getView(False) and self.hasMultipleViews() and not swtsimulator.DisplayFilter.instance.hasEvents()
-
+        return self.hasMultipleViews() and \
+            (not swtsimulator.DisplayFilter.instance.hasEvents() or \
+            swtsimulator.DisplayFilter.instance.hasEventOfType(self.getSignalsToFilter(), self.getTabFolder()))
+    
     def hasMultipleViews(self):
         # If there is only one view, don't try to record if it's activated, it's probably just been created...
         return sum((i.isActivatable() for i in self.allInstances)) > 1
@@ -213,7 +319,24 @@ class PartActivateEvent(storytext.guishared.GuiEvent):
 
     def delayLevel(self, part, *args):
         # If there are events for other shells, implies we should delay as we're in a dialog
-        return swtsimulator.DisplayFilter.instance.otherEventCount(part, [])
+        return swtsimulator.DisplayFilter.instance.otherEventCount(part, self.isTriggeringEvent)
+    
+    def isTriggeringEvent(self, event):
+        return event.widget is self.getTabFolder() and event.type in [ SWT.MouseDown, SWT.Selection ]
+    
+    def widgetDisposed(self):
+        control = self.getControl()
+        return control is None or swtsimulator.runOnUIThread(control.isDisposed)
+    
+    def widgetVisible(self):
+        return True
+        
+    def widgetSensitive(self):
+        return True
+
+    def describeWidget(self):
+        control = self.getControl()
+        return "of type " + control.__class__.__name__
     
     def getControl(self):
         return self.widget.getViewReference().getPane().getControl()
@@ -239,16 +362,18 @@ class PartActivateEvent(storytext.guishared.GuiEvent):
         return True
         
     def implies(self, stateChangeOutput, stateChangeEvent, *args):
-        return isinstance(stateChangeEvent, PartActivateEvent)
+        return isinstance(stateChangeEvent, PartActivateEvent) and stateChangeOutput.startswith(self.name)
     
     @classmethod
     def getSignalsToFilter(cls):
-        return []
+        return [ SWT.MouseDown ]
 
     @classmethod
     def getAssociatedSignal(cls, widget):
         return "ActivatePart"
     
+    
+    
 class PartCloseEvent(storytext.guishared.GuiEvent):
     def connectRecord(self, method):
         class RecordListener(IPartListener):
@@ -278,6 +403,11 @@ class PartCloseEvent(storytext.guishared.GuiEvent):
 class RCPCTabSelectEvent(swtsimulator.CTabSelectEvent):
     def implies(self, stateChangeOutput, stateChangeEvent, *args):
         return isinstance(stateChangeEvent, PartActivateEvent) or swtsimulator.CTabSelectEvent.implies(self, stateChangeOutput, stateChangeEvent, *args)
+        
+    @classmethod
+    def getSignalsToFilter(cls):
+        return [ SWT.Selection, SWT.MouseDown ] # Actually so PartActivateEvent works properly
+
 
     
 # Removing the last tab in a view folder causes the entire folder to be disposed
@@ -308,19 +438,17 @@ class RCPCTabCloseEvent(swtsimulator.CTabCloseEvent):
     
 class SWTBotExpandableComposite(AbstractSWTBotControl):
     def clickOnCenter(self):
-        firstChild = self.widget.getChildren()[0]
-        display = self.widget.getDisplay()
         # click(True) will move the mouse pointer
         # Can't find any other way to do this.
         # We can at least move it back again when we're done to avoid other bad effects
-        currPos = swtsimulator.runOnUIThread(display.getCursorLocation)
+        eventPoster = swtsimulator.EventPoster(self.widget.getDisplay())
+        eventPoster.performAndReturn(self.clickOnFirstChild)
+        
+    def clickOnFirstChild(self):        
+        firstChild = self.widget.getChildren()[0]
         SWTBotExpandableComposite(firstChild).click(True)
-        event = Event()
-        event.type = SWT.MouseMove
-        event.x = currPos.x
-        event.y = currPos.y
-        swtsimulator.runOnUIThread(display.post, event)
-
+        
+        
 class ExpandableCompositeEvent(swtsimulator.SelectEvent):
     def shouldRecord(self, *args):
         # To do this properly, we need to check MouseDown events
@@ -344,7 +472,8 @@ class ExpandableCompositeEvent(swtsimulator.SelectEvent):
 swtsimulator.eventTypes.append((swtbot.widgets.SWTBotView, [ PartActivateEvent, PartCloseEvent ]))
 swtsimulator.eventTypes.append((SWTBotExpandableComposite, [ ExpandableCompositeEvent ]))
 
-replacements = [(swtsimulator.CTabCloseEvent, RCPCTabCloseEvent),
+replacements = [(swtsimulator.SelectEvent, RCPSelectEvent),
+                (swtsimulator.CTabCloseEvent, RCPCTabCloseEvent),
                 (swtsimulator.CTabSelectEvent, RCPCTabSelectEvent)]
 
 for swtbotClass, eventClasses in swtsimulator.eventTypes:
