@@ -10,6 +10,7 @@ from org.eclipse.jface.resource import ImageDescriptor
 from array import array
 from ordereddict import OrderedDict
 
+
 class Describer(storytext.guishared.Describer):
     styleNames = [ (swt.widgets.CoolItem, []),
                    (swt.widgets.Item    , [ "SEPARATOR", "DROP_DOWN", "CHECK", "CASCADE", "RADIO" ]),
@@ -20,8 +21,8 @@ class Describer(storytext.guishared.Describer):
                    (swt.widgets.Text    , [ "PASSWORD", "SEARCH", "READ_ONLY" ]) ]
     ignoreWidgets = [ types.NoneType ]
     # DateTime children are an implementation detail
-    # Coolbars and Expandbars describe their children directly : they have two parallel children structures
-    ignoreChildren = (swt.widgets.CoolBar, swt.widgets.ExpandBar, swt.widgets.DateTime, swt.widgets.Group)
+    # Coolbars, Toolbars and Expandbars describe their children directly : they have two parallel children structures
+    ignoreChildren = (swt.widgets.CoolBar, swt.widgets.ExpandBar, swt.widgets.ToolBar, swt.widgets.DateTime, swt.widgets.Group)
     statelessWidgets = [ swt.widgets.Sash ]
     stateWidgets = [ swt.widgets.Shell, swt.widgets.Button, swt.widgets.Menu, swt.widgets.Link,
                      swt.widgets.CoolBar, swt.widgets.ToolBar, swt.widgets.Label, swt.custom.CLabel,
@@ -35,26 +36,42 @@ class Describer(storytext.guishared.Describer):
         storytext.guishared.Describer.__init__(self)
         self.canvasCounter = storytext.guishared.WidgetCounter()
         self.contextMenuCounter = storytext.guishared.WidgetCounter(self.contextMenusEqual)
+        self.customTooltipCounter = storytext.guishared.WidgetCounter(self.tooltipsEqual)
         self.widgetsAppeared = []
+        self.widgetsMoved = []
+        self.parentsResized = set()
         self.widgetsDescribed = set()
         self.browserStates = {}
         self.clipboardText = None
         self.storedImages = {}
         self.imageToName = {}
         self.handleImages()
+        self.screenshotNumber = 0
 
     def setWidgetPainted(self, widget):
         if widget not in self.widgetsDescribed and widget not in self.windows and widget not in self.widgetsAppeared:
             self.logger.debug("Widget painted " + self.getRawData(widget))
             self.widgetsAppeared.append(widget)
-
+        
     def setWidgetShown(self, widget):
         # Menu show events seem a bit spurious, they aren't really shown at this point:
         # ScrollBar shows are not relevant to anything
-        if not isinstance(widget, (swt.widgets.Menu, swt.widgets.ScrollBar)) and widget not in self.widgetsAppeared:
+        if isinstance(widget, swt.widgets.Control) and widget not in self.widgetsAppeared:
             self.logger.debug("Widget shown " + self.getRawData(widget))
             self.widgetsAppeared.append(widget)
-
+            if widget in self.widgetsMoved:
+                self.widgetsMoved.remove(widget)
+            
+    def setWidgetMoved(self, widget):
+        if isinstance(widget, swt.widgets.Control) and widget not in self.widgetsAppeared and widget.getParent() not in self.parentsResized:
+            self.logger.debug("Widget moved " + self.getRawData(widget))
+            self.widgetsMoved.append(widget)
+        
+    def setWidgetResized(self, widget):
+        if isinstance(widget, swt.widgets.Control):
+            self.parentsResized.add(widget)
+            self.parentsResized.add(widget.getParent())
+                    
     def addFilters(self, display):
         class ShowListener(swt.widgets.Listener):
             def handleEvent(listenerSelf, e):
@@ -64,29 +81,88 @@ class Describer(storytext.guishared.Describer):
             def handleEvent(listenerSelf, e):
                 storytext.guishared.catchAll(self.setWidgetPainted, e.widget)
 
+        class MoveListener(swt.widgets.Listener):
+            def handleEvent(listenerSelf, e):
+                storytext.guishared.catchAll(self.setWidgetMoved, e.widget)
+
+        class ResizeListener(swt.widgets.Listener):
+            def handleEvent(listenerSelf, e):
+                storytext.guishared.catchAll(self.setWidgetResized, e.widget)
+
         display.addFilter(swt.SWT.Show, ShowListener())
         display.addFilter(swt.SWT.Paint, PaintListener())
+        display.addFilter(swt.SWT.Move, MoveListener())
+        display.addFilter(swt.SWT.Resize, ResizeListener())
+        display.addFilter(swt.SWT.Dispose, ResizeListener()) # Being disposed is the ultimate resize :)
+
+    def getScreenshotFileName(self, screenshotDir):
+        return os.path.join(screenshotDir, "screenshot" + str(self.screenshotNumber) + ".png")
+
+    def writeScreenshot(self, shell):
+        display = shell.getDisplay()
+        gc = swt.graphics.GC(display);
+        image = swt.graphics.Image(display, shell.getBounds())
+        gc.copyArea(image, shell.getBounds().x, shell.getBounds().y)
+        gc.dispose()
         
+        imageLoader = swt.graphics.ImageLoader()
+        imageLoader.data = [ image.getImageData() ]
+        self.screenshotNumber += 1
+        screenshotDir = os.path.join(os.getenv("TEXTTEST_LOG_DIR", os.getcwd()), "screenshots")
+        if not os.path.isdir(screenshotDir):
+            os.makedirs(screenshotDir)
+        fileName = self.getScreenshotFileName(screenshotDir)
+        while os.path.isfile(fileName):
+            self.screenshotNumber += 1
+            fileName = self.getScreenshotFileName(screenshotDir)
+        imageLoader.save(fileName, swt.SWT.IMAGE_PNG) 
+    
     def describeWithUpdates(self, shellMethod):
         shell = shellMethod()
+        if self.writeScreenshots:
+            self.writeScreenshot(shell)
         if shell in self.windows:
             stateChanges = self.findStateChanges(shell)
-            stateChangeWidgets = [ widget for widget, old, new in stateChanges ]
+            stateChangeWidgets = [ widget for widget, _, _ in stateChanges ]
             if self.structureLog.isEnabledFor(logging.DEBUG):
                 for widget in stateChangeWidgets:
+                    self.structureLog.info("Widget changed state:")
                     self.describeStructure(widget)
-            describedForAppearance = self.describeAppearedWidgets(stateChangeWidgets)
+            self.processMovedWidgets()
+            describedForAppearance = self.describeAppearedWidgets(stateChangeWidgets, shell)
             self.describeStateChanges(stateChanges, describedForAppearance)
+            self.widgetsAppeared = filter(lambda w: self.validAndShowing(w) and self.inDifferentShell(w, shell), self.widgetsAppeared)
+        else:
+            self.widgetsAppeared = []
+        self.parentsResized = set()
+        self.widgetsMoved = []
+            
         if shell is not None:
             self.describeClipboardChanges(shell.getDisplay())
             self.describe(shell)
-        self.widgetsAppeared = []
+            
+    def processMovedWidgets(self):
+        # We are looking for cases of reordering: at least two widgets in the same parent must have moved for this to happen
+        moved = filter(lambda w: not w.isDisposed() and self.describeClass(w.__class__.__name__), self.widgetsMoved)
+        if len(moved) > 1:
+            self.logger.debug("Handling moved widgets " + repr(map(self.getRawData, moved)))
+            parents = [ w.getParent() for w in moved ]
+            self.logger.debug("Parents " + repr(map(self.getRawData, parents)))
+            for widget in moved:
+                if parents.count(widget.getParent()) > 1:
+                    self.widgetsAppeared.append(widget)
         
     def shouldCheckForUpdates(self, widget, shell):
         return not widget.isDisposed() and widget.getShell() == shell
     
-    def widgetShowing(self, widget):
-        return not widget.isDisposed() and util.isVisible(widget)
+    def inDifferentShell(self, widget, shell):
+        return not isinstance(widget, swt.widgets.Shell) and widget.getShell() != shell
+    
+    def validAndShowing(self, widget):
+        return not widget.isDisposed() and util.isVisible(widget) 
+
+    def widgetShowing(self, widget, shell):
+        return self.validAndShowing(widget) and not self.inDifferentShell(widget, shell)
 
     def describeClipboardChanges(self, display):
         clipboard = swt.dnd.Clipboard(display)
@@ -116,25 +192,30 @@ class Describer(storytext.guishared.Describer):
         return shell.getText()
 
     def getAllItemDescriptions(self, itemBar, indent=0, subItemMethod=None,
-                               prefix="", selection=[], columnCount=0, **kw):
+                               prefix="", selection=[], columnCount=0, enclosingJfaceTooltip=None, **kw):
         descs = []
         for item in itemBar.getItems():
             currPrefix = prefix + " " * indent * 2
             selected = item in selection or (hasattr(item, "getSelection") and item.getSelection())
             if columnCount:
-                row = [ self.getItemColumnDescription(item, i, currPrefix, selected) for i in range(columnCount) ]
+                row = [ self.getItemColumnDescription(item, i, currPrefix, selected, enclosingJfaceTooltip) for i in range(columnCount) ]
                 descs.append(row)
             else:
-                itemDesc = self.getItemDescription(item, currPrefix, selected)
+                itemDesc = self.getItemDescription(item, currPrefix, selected, enclosingJfaceTooltip)
                 if itemDesc:
                     descs.append(itemDesc)
             if subItemMethod:
-                descs += subItemMethod(item, indent, prefix=prefix, selection=selection, columnCount=columnCount, **kw)
+                descs += subItemMethod(item, indent, prefix=prefix, selection=selection, 
+                                       columnCount=columnCount, enclosingJfaceTooltip=enclosingJfaceTooltip, **kw)
         return descs
 
-    def getCascadeMenuDescriptions(self, item, indent, storeStatesForSubMenus=False, **kw):
+    def getCascadeMenuDescriptions(self, item, indent, storeStatesForSubMenus=False, describeMenus=None, **kw):
         cascadeMenu = item.getMenu()
         if cascadeMenu:
+            if describeMenus:
+                text = item.getText().replace("&", "")
+                if text not in describeMenus:
+                    return []
             descs = self.getAllItemDescriptions(cascadeMenu, indent+1, subItemMethod=self.getCascadeMenuDescriptions, 
                                                 storeStatesForSubMenus=storeStatesForSubMenus, **kw)
             if indent == 1 and storeStatesForSubMenus:
@@ -151,26 +232,25 @@ class Describer(storytext.guishared.Describer):
 
     def getExpandItemDescriptions(self, item, indent, *args, **kw):
         if item.getExpanded():
-            return [ self.getItemControlDescription(item, indent + 1, *args, **kw) ]
+            return self.getCoolItemDescriptions(item, indent + 1)
+        else:
+            return []
+        
+    def getToolItemControls(self, item, indent, **kw):
+        control = item.getControl()
+        if control:
+            return [ (control, indent) ] 
         else:
             return []
 
     def getCoolItemDescriptions(self, item, *args, **kw):
-        itemDesc = self.getItemControlDescription(item, *args, **kw)
-        if itemDesc:
-            return [ itemDesc ] 
-        else:
-            return []
-
-    def getItemControlDescription(self, item, indent, **kw):
-        control = item.getControl()
-        if control:
-            descLines = self.getDescription(control).splitlines()
-            paddedLines = [ " " * indent * 2 + line for line in descLines ]
-            return "\n".join(paddedLines) + "\n"
-        else:
-            return ""
+        return [ self.getItemControlDescription(c, i) for c, i in self.getToolItemControls(item, *args, **kw) ]
 
+    def getItemControlDescription(self, control, indent):
+        descLines = self.getDescription(control).splitlines()
+        paddedLines = [ " " * indent * 2 + line for line in descLines ]
+        return "\n".join(paddedLines) + "\n"
+        
     def getMenuDescription(self, menu, indent=1, **kw):
         return self.getItemBarDescription(menu, indent=indent, subItemMethod=self.getCascadeMenuDescriptions, **kw)
     
@@ -178,8 +258,9 @@ class Describer(storytext.guishared.Describer):
         return self.getMenuDescription(menu, indent=2)
     
     def getMenuBarDescription(self, menubar):
-        if menubar and "Menu" not in self.excludeClassNames:
-            return "Menu Bar:\n" + self.getMenuDescription(menubar, storeStatesForSubMenus=True)
+        if menubar and self.describeClass("Menu"):
+            describeMenus = self.excludeClassNames.get("Menu")
+            return "Menu Bar:\n" + self.getMenuDescription(menubar, storeStatesForSubMenus=True, describeMenus=describeMenus)
         else:
             return ""
 
@@ -191,11 +272,21 @@ class Describer(storytext.guishared.Describer):
     def getExpandBarState(self, expandbar):
         return expandbar.getChildren(), [ item.getExpanded() for item in expandbar.getItems() ] 
 
+    def itemStateToString(self, itemState):
+        if isinstance(itemState, (str, unicode)):
+            return itemState
+        else:
+            return self.getItemControlDescription(*itemState)
+
     def getToolBarDescription(self, toolbar):
-        return self.getAndStoreState(toolbar)
+        itemStates = self.getToolBarState(toolbar)
+        self.widgetsWithState[toolbar] = itemStates
+        descs = map(self.itemStateToString, itemStates)
+        return "\n".join(descs)
 
     def getToolBarState(self, toolbar):
-        return "Tool Bar:\n" + self.getItemBarDescription(toolbar, indent=1)
+        return [ "Tool Bar:" ] + self.getAllItemDescriptions(toolbar, indent=1, 
+                                                             subItemMethod=self.getToolItemControls)
     
     def getCoolBarDescription(self, coolbar):
         return "Cool Bar:\n" + self.getItemBarDescription(coolbar, indent=1,
@@ -208,6 +299,11 @@ class Describer(storytext.guishared.Describer):
     def imagesEqual(self, image1, image2):
         return image1.getImageData().data == image2.getImageData().data
 
+    def tooltipsEqual(self, data1, data2):
+        tip1, widget1 = data1
+        tip2, widget2 = data2
+        return tip1 == tip2 and widget1 == widget2
+
     def getImageDescription(self, image):
         # Seems difficult to get any sensible image information out, there is
         # basically no query API for this in SWT
@@ -297,6 +393,15 @@ class Describer(storytext.guishared.Describer):
             return prefix + desc
         else:
             return desc
+        
+    def hasPrivateMethod(self, obj, methodName):
+        return any((method.getName() == methodName for method in obj.getClass().getDeclaredMethods())) 
+        
+    def getJfaceTooltip(self, item):
+        for listener in item.getListeners(swt.SWT.MouseHover):
+            tooltip = self.getEnclosingInstance(listener)
+            if self.hasPrivateMethod(tooltip, "createToolTipContentArea"):
+                return tooltip
 
     def getControlDecorations(self, item):
         decorations = []
@@ -333,26 +438,42 @@ class Describer(storytext.guishared.Describer):
                 if desc:
                     text += "\n'" + desc + "'"
                 return text
-
+            
     def decorationVisible(self, deco):
         if hasattr(deco, "isVisible"): # added in 3.6
             return deco.isVisible()
+        else:
+            # Workaround for reflection bug in Jython 2.5.1
+            # args = (None,) if sys.version_info[:3] <= (2, 5, 1) else ()
+            # Jython 2.5.2 doesn't work anyway so we don't include this fix for now
+            return util.callPrivateMethod(deco, "shouldShowDecoration")
 
-        # Workaround for reflection bug in Jython 2.5.1
-        args = (None,) if sys.version_info[:3] <= (2, 5, 1) else ()
-        method = deco.getClass().getDeclaredMethod("shouldShowDecoration", *args)
-        method.setAccessible(True)
-        return method.invoke(deco, *args)
+    def isCustomTooltip(self, jfaceTooltip):
+        return not jfaceTooltip.__class__.__module__.startswith("org.eclipse.jface")
 
-    def getPropertyElements(self, item, selected=False):
+    def getToolTipText(self, item, jfaceTooltip):
+        if hasattr(item, "getToolTipText") and item.getToolTipText():
+            return item.getToolTipText()
+        elif jfaceTooltip and not self.isCustomTooltip(jfaceTooltip):
+            event = self.makeToolTipEvent(item)
+            if util.callPrivateMethod(jfaceTooltip, "shouldCreateToolTip", [ event ]):
+                return util.callPrivateMethod(jfaceTooltip, "getText", [ event ])
+        
+    def getPropertyElements(self, *args, **kw):
+        return self.getPropertyElementsAndTooltip(*args, **kw)[0]
+        
+    def getPropertyElementsAndTooltip(self, item, selected=False, enclosingJfaceTooltip=None):
         elements = []
         decoText = self.getControlDecorationDescription(item)
         if decoText:
             elements.append(decoText)
         if isinstance(item, swt.widgets.Spinner):
             elements += self.getSpinnerPropertyElements(item)
-        if hasattr(item, "getToolTipText") and item.getToolTipText():
-            elements.append("Tooltip '" + item.getToolTipText() + "'")
+            
+        jfaceTooltip = enclosingJfaceTooltip if isinstance(item, swt.widgets.Item) else self.getJfaceTooltip(item)
+        tooltipText = self.getToolTipText(item, jfaceTooltip)
+        if tooltipText:
+            elements.append("Tooltip '" + tooltipText + "'")
         elements += self.getStyleDescriptions(item)
         if hasattr(item, "getImage") and item.getImage():
             elements.append(self.getImageDescription(item.getImage()))
@@ -361,9 +482,12 @@ class Describer(storytext.guishared.Describer):
         if selected:
             elements.append("selected")
         elements.append(self.getContextMenuReference(item))
+        customTooltipText = self.getCustomTooltipReference(item, jfaceTooltip)
+        if customTooltipText:
+            elements.append(customTooltipText)
         if hasattr(item, "getItemCount") and hasattr(item, "getExpanded") and item.getItemCount() > 0 and not item.getExpanded():
             elements.append("+")
-        return elements
+        return elements, jfaceTooltip
 
     def getSpinnerPropertyElements(self, item):
         elements = []
@@ -439,26 +563,30 @@ class Describer(storytext.guishared.Describer):
             return url
         # Ignore non-breaking spaces, they are invisible anyway
         # Webkit returns them in invalid format without the semicolons... handle that too.
-        rawText = widget.getText().replace(u"\xa0", " ").replace("&nbsp;", " ").replace("&nbsp", " ")
-        text = storytext.guishared.encodeToLocale(rawText)
+        text = widget.getText().replace(u"\xa0", " ").replace("&nbsp;", " ").replace("&nbsp", " ")
         return BrowserHtmlParser().parse(text)
         
     def getUpdatePrefix(self, widget, oldState, state):
         if isinstance(widget, (self.getTextEntryClass(), swt.browser.Browser, swt.widgets.Spinner)):
-            return "\nUpdated " + (util.getTextLabel(widget, useContext=True) or "Text") +  " Field\n"
-        elif isinstance(widget, swt.widgets.Combo):
+            return "\nUpdated " + (util.getTextLabel(widget, useContext=True) or self.getShortWidgetIdentifier(widget) or "Text") +  " Field\n"
+        elif isinstance(widget, (swt.widgets.Combo, swt.custom.CCombo)):
             return "\nUpdated " + util.getTextLabel(widget, useContext=True) + " Combo Box\n"
         elif util.getTopControl(widget) or isinstance(widget, swt.widgets.Group):
             return "\n"
         elif isinstance(widget, swt.widgets.Menu):
             parentItem = widget.getParentItem()
+            menuRefNr = self.contextMenuCounter.getWidgetNumber(widget)
+            menuRefNr = " " + str(menuRefNr) if menuRefNr > 0 else ""
             menuName = parentItem.getText() if parentItem else "Context"
-            return "\nUpdated " + menuName + " Menu:\n"
+            return "\nUpdated " + menuName + " Menu" + menuRefNr +":\n"
         elif isinstance(widget, (swt.widgets.Label, swt.custom.CLabel)) and len(state) == 0:
             return "\nLabel now empty, previously " + oldState
         else:
             return "\nUpdated "
 
+    def getShortWidgetIdentifier(self, widget):
+        return widget.getData("org.eclipse.swtbot.widget.key")
+
     def getState(self, widget):
         if widget.isDisposed():
             # Will be caught, and the widget cleaned up
@@ -532,14 +660,22 @@ class Describer(storytext.guishared.Describer):
             return "Context Menu " + self.contextMenuCounter.getId(widget.getMenu())
         else:
             return ""
+        
+    def getCustomTooltipReference(self, item, jfaceTooltip):
+        if "CustomTooltip" not in self.excludeClassNames and jfaceTooltip and self.isCustomTooltip(jfaceTooltip):
+            itemTooltip = self.hasPrivateMethod(jfaceTooltip, "createViewerToolTipContentArea")
+            isItem = isinstance(item, swt.widgets.Item)
+            if isItem == itemTooltip:
+                return "Custom Tooltip " + self.customTooltipCounter.getId((jfaceTooltip, item))
 
     def getTreeState(self, widget):
         columns = widget.getColumns()
         columnCount = len(columns)
-        text = self.combineElements([ "Tree" ] + self.getPropertyElements(widget)) + " :\n"
+        props, jfaceTooltip = self.getPropertyElementsAndTooltip(widget)
+        text = self.combineElements([ "Tree" ] + props) + " :\n"
         rows = self.getAllItemDescriptions(widget, indent=0, subItemMethod=self.getSubTreeDescriptions,
                                            prefix="-> ", selection=widget.getSelection(),
-                                           columnCount=columnCount)
+                                           columnCount=columnCount, enclosingJfaceTooltip=jfaceTooltip)
         if columnCount > 0:
             rows.insert(0, [ c.getText() for c in columns ])
             text += str(GridFormatter(rows, columnCount))
@@ -550,22 +686,34 @@ class Describer(storytext.guishared.Describer):
     def getTableState(self, widget):
         columns = widget.getColumns()
         columnCount = len(columns)
-        text = self.combineElements([ "Table" ] + self.getPropertyElements(widget)) + " :\n"
+        props, jfaceTooltip = self.getPropertyElementsAndTooltip(widget)
+        text = self.combineElements([ "Table" ] + props) + " :\n"
         rows = self.getAllTableItemDescriptions(widget, indent=0, 
-                                           selection=widget.getSelection(),
-                                           columnCount=columnCount)
-        headerRow = [ c.getText() for c in columns if c.getWidth() > 0] # Don't show hidden columns
-        return text + self.formatTable(headerRow, rows, columnCount)
+                                                selection=widget.getSelection(),
+                                                columnCount=columnCount,
+                                                enclosingJfaceTooltip=jfaceTooltip)
+        sortColumn = widget.getSortColumn()
+        if widget.getSortDirection() == swt.SWT.UP:
+            sortDirection = "(->)"
+        elif widget.getSortDirection() == swt.SWT.DOWN:
+            sortDirection = "(<-)"
+        else:
+            sortDirection = ""
+        headerRow = [ c.getText() + sortDirection  if c == sortColumn else c.getText() for c in columns if c.getWidth() > 0] # Don't show hidden columns
+        return text + self.formatTable(headerRow, rows, max(1, columnCount))
 
     def getAllTableItemDescriptions(self, widget, indent=0,
-                               prefix="", selection=[], columnCount=0, **kw):
+                                    prefix="", selection=[], columnCount=0, enclosingJfaceTooltip=None):
         descs = []
         for item in widget.getItems():
             currPrefix = prefix + " " * indent * 2
             selected = item in selection
             if columnCount:
-                row = [ self.getItemColumnDescription(item, i, currPrefix, selected) for i in range(columnCount) if widget.getColumn(i).getWidth() > 0]
+                row = [ self.getItemColumnDescription(item, i, currPrefix, selected, enclosingJfaceTooltip) 
+                        for i in range(columnCount) if widget.getColumn(i).getWidth() > 0 ]
                 descs.append(row)
+            else:
+                descs.append([ self.getItemDescription(item, currPrefix, selected, enclosingJfaceTooltip) ])
         return descs
 
     def getTabFolderDescription(self, widget):
@@ -597,23 +745,15 @@ class Describer(storytext.guishared.Describer):
         header = "." * 6 + " " + widget.getText() + " " + "." * 6
         footer = "." * len(header)
         compositeDesc = self.getCompositeDescription(widget) or self.formatChildrenDescription(widget)
-        return header + "\n" + compositeDesc + "\n" + footer
+        return header + "\n" + str(compositeDesc) + "\n" + footer
 
     def getStateControlDescription(self, widget):
         stateControlInfo = self.getState(widget)
         stateControl = stateControlInfo[0] if isinstance(stateControlInfo, tuple) else stateControlInfo
         if stateControlInfo:
             self.widgetsWithState[widget] = stateControlInfo
-        if stateControl:
-            if len(widget.getChildren()) > 1:
-                header = "+" * 6 + " Stacked Layout " + "+" * 6
-                footer = "+" * len(header)
-                return header + "\n" + self.getDescription(stateControl) + "\n" + footer
-            else:
-                return self.getDescription(stateControl)
-        else:
-            return ""
-
+        return self.getDescription(stateControl) if stateControl else ""
+        
     def getVerticalDividePositions(self, children):
         positions = []
         for child in children:
@@ -625,13 +765,19 @@ class Describer(storytext.guishared.Describer):
         layout = widget.getLayout()
         return layout is not None and isinstance(layout, (swt.layout.GridLayout, swt.layout.FillLayout,
                                                           swt.layout.RowLayout, swt.custom.StackLayout))
-
+    
     def _getDescription(self, widget):
         self.widgetsDescribed.add(widget)
         desc = storytext.guishared.Describer._getDescription(self, widget)
         if desc and isinstance(widget, (swt.widgets.ExpandBar, swt.widgets.Tree, swt.widgets.List, swt.widgets.Table)):
-            desc = str(desc) + self.formatContextMenuDescriptions()
+            desc = unicode(desc) + self.formatContextMenuDescriptions()
         return desc
+    
+    def handleGridFormatter(self, formatter):
+        output = storytext.guishared.Describer.handleGridFormatter(self, formatter)
+        if isinstance(output, (str, unicode)):
+            output += self.formatContextMenuDescriptions()
+        return output
 
     def getWindowContentDescription(self, shell):
         desc = ""
@@ -666,9 +812,40 @@ class Describer(storytext.guishared.Describer):
                 if not contextMenu.isDisposed():
                     menuDesc = self.getMenuDescription(contextMenu)
                     text += "\n\nContext Menu " + str(menuId) + ":\n" + menuDesc
-                    self.widgetsWithState[contextMenu] = self.getMenuState(contextMenu) 
+                    self.widgetsWithState[contextMenu] = self.getMenuState(contextMenu)
+        if "CustomTooltip" not in self.excludeClassNames:
+            for (tooltip, widget), tooltipId in self.customTooltipCounter.getWidgetsForDescribe():
+                text += "\n\nCustom Tooltip " + str(tooltipId) + ":\n" + self.getCustomTooltipDescription(tooltip, widget)
         return text
+    
+    def makeToolTipEvent(self, widgetOrItem):
+        event = swt.widgets.Event()
+        event.type = swt.SWT.MouseHover
+        if isinstance(widgetOrItem, swt.widgets.Item):
+            event.widget = widgetOrItem.getParent()
+            event.item = widgetOrItem
+            bounds = widgetOrItem.getBounds()
+            event.x = util.getInt(bounds.x) + util.getInt(bounds.width) / 2
+            event.y = util.getInt(bounds.y) + util.getInt(bounds.height) / 2
+        else:
+            event.widget = widgetOrItem
+            event.item = None
+            event.x = -1
+            event.y = -1
+        return event
 
+    def getCustomTooltipDescription(self, tooltip, widget):
+        event = self.makeToolTipEvent(widget)
+        if util.callPrivateMethod(tooltip, "shouldCreateToolTip", [ event ]):
+            shell = swt.widgets.Shell()
+            result = util.callPrivateMethod(tooltip, "createToolTipContentArea", [ event, shell ], [ swt.widgets.Event, swt.widgets.Composite ])
+            desc = self.getDescription(result)
+            result.dispose()
+            shell.dispose()
+            return desc
+        else:
+            return ""
+        
     def getHorizontalSpan(self, widget, columns):
         layout = widget.getLayoutData()
         if hasattr(layout, "horizontalSpan"):
@@ -735,11 +912,14 @@ class Describer(storytext.guishared.Describer):
     def checkWindow(self, window):
         # Don't describe tooltips
         return (window.getStyle() & swt.SWT.TOOL) == 0
-    
+
+    def splitState(self, state):
+        return state if isinstance(state, list) else state.split("\n")
+
     def getStateChangeDescription(self, widget, oldState, state):
         if isinstance(widget, (swt.widgets.Menu, swt.widgets.ToolBar)):
-            old = oldState.split("\n")
-            new = state.split("\n")
+            old = self.splitState(oldState)
+            new = self.splitState(state)
             if len(old) == len(new):
                 return self.getDiffedDescription(widget, old, new)
         return storytext.guishared.Describer.getStateChangeDescription(self, widget, oldState, state)
