@@ -1,8 +1,9 @@
 
 """ Generic recorder classes. GUI-specific stuff is in guishared.py """
 
-import os, sys, signal, time, logging, re
-from threading import Thread, Timer
+import os, sys, signal, time, re
+import encodingutils
+from threading import Thread, Timer, Lock
 from definitions import *
 from copy import copy
 
@@ -38,8 +39,8 @@ class ReplayScript(object):
         self.name = scriptName
         if not os.path.isfile(scriptName):
             raise UseCaseScriptError, "Cannot replay script " + repr(scriptName) + ", no such file or directory."
-        for line in open(scriptName):
-            line = line.strip()
+        for line in encodingutils.openEncoded(scriptName):
+            line = line.strip("\r\n")
             if not ignoreComments or (line != "" and line[0] != "#"):
                 self.commands.append(line)
                 
@@ -53,13 +54,21 @@ class ReplayScript(object):
         self.exitObservers.append(observer)
 
     def getShortcutName(self):
-        return os.path.basename(self.name).split(".")[0].replace("_", " ").replace("#", "_")
+        return self.tryToGetName(self.name)
+
+    @classmethod
+    def tryToGetName(cls, text):
+        return os.path.basename(text).split(".")[0].replace("_", " ").replace("#", "_")
 
     def getShortcutRegexp(self):
         return self.getRegexp(self.getShortcutName())
 
     def getShortcutNameWithArgs(self, args):
-        splittedName = self.getShortcutName().lower().split("$")
+        return self.getTextWithArgs(self.getShortcutName().lower(), args)
+        
+    @staticmethod
+    def getTextWithArgs(text, args):
+        splittedName = text.split("$")
         if len(splittedName) == 1:
             return splittedName[0]
         name = ""
@@ -70,7 +79,8 @@ class ReplayScript(object):
                 name += part
         return name
 
-    def transformToRegexp(self, text):
+    @classmethod
+    def transformToRegexp(cls, text):
         for char in "^[]{}*?|+()":
             text = text.replace(char, "\\" + char)
         # handle numbered variables
@@ -173,10 +183,15 @@ class ReplayScript(object):
                 if newEvent in currEvents:
                     currEvents[newEvent] += count
                 else:
-                    currEvents[newEvent] = 1
+                    currEvents[newEvent] = count
             self.commands[-1] = assembleWaitCommand(currEvents.items())
         else:
             self.commands.append(command)
+    
+    def rename(self, newName):
+        newPath = os.path.join(os.path.dirname(self.name), newName)
+        os.rename(self.name, newPath)
+        self.name = newPath
         
         
 class ShortcutManager:
@@ -213,13 +228,24 @@ class ShortcutManager:
         argLength2 = sum(map(len, args2))
         return argLength1 < argLength2
     
+    def remove(self, shortcut):
+        self.shortcuts.remove((shortcut.getShortcutRegexp(), shortcut))
+        os.remove(shortcut.name)
+        
+    def rename(self, oldName, newName):
+        shortcut = self.findShortcut(oldName)[0]
+        self.shortcuts.remove((shortcut.getShortcutRegexp(), shortcut))
+        shortcut.rename(newName)
+        self.add(shortcut)
+    
     
 class UseCaseReplayer:
     def __init__(self, timeout=60):
-        self.logger = logging.getLogger("storytext replay log")
+        self.logger = encodingutils.getEncodedLogger("storytext replay log")
         self.scripts = []
         self.shortcutManager = ShortcutManager()
         self.events = {}
+        self.appEventLock = Lock()
         self.waitingForEvents = []
         self.applicationEventNames = set()
         self.replayThread = None
@@ -243,17 +269,17 @@ class UseCaseReplayer:
     def getShortcuts(self):
         return self.shortcutManager.getShortcuts()
     
-    def addEvent(self, event):
-        self.events.setdefault(event.name, []).append(event)
+    def addEvent(self, event, eventNames):
+        for name in eventNames:
+            self.events.setdefault(name, []).append(event)
     
-
     def writeRecursiveError(self, script, arguments):
         sys.stderr.write("ERROR: Cannot execute shortcut command '" + script.getShortcutNameWithArgs(arguments) + "' - shortcut is trying to call itself!\n")
 
     def addScript(self, script, arguments=[], enableReading=False):
         if self.isNonRecursive(script):
             self.scripts.append((script, arguments))
-            self.runScript(script, enableReading)
+            self.runScript(script, arguments, enableReading)
             return True
         else:
             self.writeRecursiveError(script, arguments)
@@ -264,14 +290,15 @@ class UseCaseReplayer:
 
     def tryRunScript(self):
         if self.isActive():
-            script, _ = self.scripts[-1]
-            self.runScript(script, enableReading=True)
+            script, args = self.scripts[-1]
+            self.runScript(script, args, enableReading=True)
 
-    def runScript(self, script, enableReading):
+    def runScript(self, script, arguments, enableReading):
         if self.shortcutManager.shortcuts:
-            scriptCommand = script.getCommand(matching=self.shortcutManager.getRegexps())
+            scriptCommand = script.getCommand(arguments, matching=self.shortcutManager.getRegexps())
             if scriptCommand:
                 newScript, args = self.shortcutManager.findShortcut(scriptCommand)
+                self.logger.debug("Found initial shortcut '" + newScript.getShortcutName() + "' with args " + repr(args))
                 if self.addScript(newScript, args, enableReading):
                     return
             
@@ -331,6 +358,7 @@ class UseCaseReplayer:
     def registerApplicationEvent(self, eventName, timeDelay):
         origEventName = eventName
         count = 2
+        self.appEventLock.acquire()
         while eventName in self.applicationEventNames:
             eventName = origEventName + " * " + str(count)
             count += 1
@@ -339,6 +367,7 @@ class UseCaseReplayer:
         self.timeDelayNextCommand = timeDelay
         if len(self.waitingForEvents) > 0 and self.waitingCompleted():
             self.completedApplicationEvents()
+        self.appEventLock.release()
             
     def applicationEventRename(self, oldName, newName):
         toRename = filter(lambda eventName: oldName in eventName and newName not in eventName,
@@ -458,7 +487,7 @@ class UseCaseReplayer:
         except UseCaseScriptError:
             # We don't terminate scripts if they contain errors
             type, value, _ = sys.exc_info()
-            self.write("ERROR: Could not simulate command " + repr(command) + " - " + str(value))
+            self.write("ERROR: Could not simulate command '" + command + "' - " + unicode(value))
             
     def describeEvent(self, commandName, argumentString):
         self.write("")
@@ -497,7 +526,7 @@ class UseCaseReplayer:
         if scriptCommand.strip() == commandName:
             return ""
         else:
-            return scriptCommand.replace(commandName + " ", "").rstrip()
+            return scriptCommand.replace(commandName + " ", "").rstrip("\r\n")
 
     def findCommandName(self, command):
         if command.startswith(signalCommandName):
@@ -526,12 +555,14 @@ class UseCaseReplayer:
     def processWait(self, applicationEventStr):
         eventsToWaitFor = applicationEventStr.split(", ")
         self.describeAppEventsWaiting(eventsToWaitFor)
+        self.appEventLock.acquire()
         allEventsToWaitFor = self.waitingForEvents + eventsToWaitFor
         # Must make sure this is atomic - don't add events one at a time with +=
         self.waitingForEvents = allEventsToWaitFor
         complete = self.waitingCompleted()
         if not complete:
             self.setAppEventTimer()
+        self.appEventLock.release()
         return complete
     
     def processSignalCommand(self, signalArg):
