@@ -21,7 +21,7 @@ class ScriptEngine:
     def __init__(self, enableShortcuts=False, **kwargs):
         os.environ["STORYTEXT_HOME"] = self.storytextHome
         self.enableShortcuts = enableShortcuts
-        self.recorder = recorder.UseCaseRecorder()
+        self.recorder = recorder.UseCaseRecorder(self.getShortcuts())
         self.replayer = self.createReplayer(**kwargs)
         self.registerShortcuts()
         self.replayer.tryRunScript()
@@ -39,11 +39,10 @@ class ScriptEngine:
         for shortcut in self.getShortcuts():
             if self.replayerActive():
                 self.replayer.registerShortcut(shortcut)
-            if self.recorderActive():
-                self.recorder.registerShortcut(shortcut)
 
-    def getShortcuts(self, storyTextHome=None):
-        home = storyTextHome if storyTextHome else self.storytextHome
+    @classmethod
+    def getShortcuts(cls, storyTextHome=None):
+        home = storyTextHome if storyTextHome else cls.storytextHome
         shortcuts = []
         if not os.path.isdir(home):
             return shortcuts
@@ -74,6 +73,10 @@ class ScriptEngine:
         if self.recorderActive():
             self.recorder.applicationEventDelay(name, **kw)
 
+    def applicationEventRemove(self, *args, **kw):
+        if self.recorderActive():
+            self.recorder.unregisterApplicationEvent(*args, **kw)
+
     def run(self, options, args):
         if len(args) == 0:
             return False
